package javax.swing.text;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import javax.swing.JFormattedTextField;
import java.text.ParseException;

public class MaskFormatter extends DefaultFormatter
{
    private static final char DIGIT_KEY = '#';
    private static final char LITERAL_KEY = '\'';
    private static final char UPPERCASE_KEY = 'U';
    private static final char LOWERCASE_KEY = 'L';
    private static final char ALPHA_NUMERIC_KEY = 'A';
    private static final char CHARACTER_KEY = '?';
    private static final char ANYTHING_KEY = '*';
    private static final char HEX_KEY = 'H';
    private static final MaskFormatter.MaskCharacter[] EmptyMaskChars;
    private String mask;
    private transient MaskFormatter.MaskCharacter[] maskChars;
    private String validCharacters;
    private String invalidCharacters;
    private String placeholderString;
    private char placeholder;
    private boolean containsLiteralChars;
    
    public MaskFormatter() {
        this.setAllowsInvalid(false);
        this.containsLiteralChars = true;
        this.maskChars = MaskFormatter.EmptyMaskChars;
        this.placeholder = ' ';
    }
    
    public MaskFormatter(String mask) throws ParseException {
        this();
        this.setMask(mask);
    }
    
    public void setMask(String mask) throws ParseException {
        this.mask = mask;
        this.updateInternalMask();
    }
    
    public String getMask() {
        return this.mask;
    }
    
    public void setValidCharacters(String validCharacters) {
        this.validCharacters = validCharacters;
    }
    
    public String getValidCharacters() {
        return this.validCharacters;
    }
    
    public void setInvalidCharacters(String invalidCharacters) {
        this.invalidCharacters = invalidCharacters;
    }
    
    public String getInvalidCharacters() {
        return this.invalidCharacters;
    }
    
    public void setPlaceholder(String placeholderString) {
        this.placeholderString = placeholderString;
    }
    
    public String getPlaceholder() {
        return this.placeholderString;
    }
    
    public void setPlaceholderCharacter(char placeholder) {
        this.placeholder = placeholder;
    }
    
    public char getPlaceholderCharacter() {
        return this.placeholder;
    }
    
    public void setValueContainsLiteralCharacters(boolean containsLiteralChars) {
        this.containsLiteralChars = containsLiteralChars;
    }
    
    public boolean getValueContainsLiteralCharacters() {
        return this.containsLiteralChars;
    }
    
    @Override
    public Object stringToValue(String s) throws ParseException {
        return this.stringToValue(s, true);
    }
    
    @Override
    public String valueToString(Object o) throws ParseException {
        String s = (o == null) ? "" : o.toString();
        StringBuilder sb = new StringBuilder();
        this.append(sb, s, new int[] { 0 }, this.getPlaceholder(), this.maskChars);
        return sb.toString();
    }
    
    @Override
    public void install(JFormattedTextField formattedTextField) {
        super.install(formattedTextField);
        if (formattedTextField != null) {
            Object value = formattedTextField.getValue();
            try {
                this.stringToValue(this.valueToString(value));
            }
            catch (ParseException ex) {
                this.setEditValid(false);
            }
        }
    }
    
    private Object stringToValue(String stripLiteralChars, boolean b) throws ParseException {
        int invalidOffset;
        if ((invalidOffset = this.getInvalidOffset(stripLiteralChars, b)) == -1) {
            if (!this.getValueContainsLiteralCharacters()) {
                stripLiteralChars = this.stripLiteralChars(stripLiteralChars);
            }
            return super.stringToValue(stripLiteralChars);
        }
        else {
            throw new ParseException("stringToValue passed invalid value", invalidOffset);
        }
    }
    
    private int getInvalidOffset(String s, boolean b) {
        int length = s.length();
        if (length != this.getMaxLength()) {
            return length;
        }
        int i = 0;
        while (i < s.length()) {
            int char1 = s.charAt(i);
            if (!this.isValidCharacter(i, (char)char1) && (b || !this.isPlaceholder(i, (char)char1))) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    private void append(StringBuilder sb, String s, int[] array, String s2, MaskFormatter.MaskCharacter[] array2) throws ParseException {
        for (int i = 0; i < array2.length; ++i) {
            array2[i].append(sb, s, array, s2);
        }
    }
    
    private void updateInternalMask() throws ParseException {
        String mask = this.getMask();
        ArrayList list = new ArrayList();
        if (mask != null) {
            for (int i = 0, length = mask.length(); i < length; ++i) {
                int char1 = mask.charAt(i);
                switch (char1) {
                    case 35: {
                        list.add(new MaskFormatter.DigitMaskCharacter());
                        break;
                    }
                    case 39: {
                        if (++i < length) {
                            list.add(new MaskFormatter.LiteralCharacter(mask.charAt(i)));
                            break;
                        }
                        break;
                    }
                    case 85: {
                        list.add(new MaskFormatter.UpperCaseCharacter());
                        break;
                    }
                    case 76: {
                        list.add(new MaskFormatter.LowerCaseCharacter());
                        break;
                    }
                    case 65: {
                        list.add(new MaskFormatter.AlphaNumericCharacter());
                        break;
                    }
                    case 63: {
                        list.add(new MaskFormatter.CharCharacter());
                        break;
                    }
                    case 42: {
                        list.add(new MaskFormatter.MaskCharacter());
                        break;
                    }
                    case 72: {
                        list.add(new MaskFormatter.HexCharacter());
                        break;
                    }
                    default: {
                        list.add(new MaskFormatter.LiteralCharacter((char)char1));
                        break;
                    }
                }
            }
        }
        if (list.size() == 0) {
            this.maskChars = MaskFormatter.EmptyMaskChars;
        }
        else {
            list.toArray(this.maskChars = new MaskFormatter.MaskCharacter[list.size()]);
        }
    }
    
    private MaskFormatter.MaskCharacter getMaskCharacter(int n) {
        if (n >= this.maskChars.length) {
            return null;
        }
        return this.maskChars[n];
    }
    
    private boolean isPlaceholder(int n, char c) {
        return this.getPlaceholderCharacter() == c;
    }
    
    private boolean isValidCharacter(int n, char c) {
        return this.getMaskCharacter(n).isValidCharacter(c);
    }
    
    private boolean isLiteral(int n) {
        return this.getMaskCharacter(n).isLiteral();
    }
    
    private int getMaxLength() {
        return this.maskChars.length;
    }
    
    private char getLiteral(int n) {
        return this.getMaskCharacter(n).getChar('\0');
    }
    
    private char getCharacter(int n, char c) {
        return this.getMaskCharacter(n).getChar(c);
    }
    
    private String stripLiteralChars(String s) {
        Object o = null;
        int beginIndex = 0;
        for (int i = 0; i < s.length(); ++i) {
            if (this.isLiteral(i)) {
                if (o == null) {
                    o = new StringBuilder();
                    if (i > 0) {
                        o.append(s.substring(0, i));
                    }
                }
                else if (beginIndex != i) {
                    o.append(s.substring(beginIndex, i));
                }
                beginIndex = i + 1;
            }
        }
        if (o == null) {
            return s;
        }
        if (beginIndex != s.length()) {
            if (o == null) {
                return s.substring(beginIndex);
            }
            o.append(s.substring(beginIndex));
        }
        return o.toString();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        try {
            this.updateInternalMask();
        }
        catch (ParseException ex) {}
    }
    
    @Override
    boolean isNavigatable(int n) {
        return this.getAllowsInvalid() || (n < this.getMaxLength() && !this.isLiteral(n));
    }
    
    @Override
    boolean isValidEdit(DefaultFormatter.ReplaceHolder replaceHolder) {
        if (!this.getAllowsInvalid()) {
            String replaceString = this.getReplaceString(replaceHolder.offset, replaceHolder.length, replaceHolder.text);
            try {
                replaceHolder.value = this.stringToValue(replaceString, false);
                return true;
            }
            catch (ParseException ex) {
                return false;
            }
        }
        else {
            return true;
        }
    }
    
    @Override
    boolean canReplace(DefaultFormatter.ReplaceHolder replaceHolder) {
        if (!this.getAllowsInvalid()) {
            Object o = null;
            String text = replaceHolder.text;
            int a = (text != null) ? text.length() : 0;
            if (a == 0 && replaceHolder.length == 1 && this.getFormattedTextField().getSelectionStart() != replaceHolder.offset) {
                while (replaceHolder.offset > 0 && this.isLiteral(replaceHolder.offset)) {
                    --replaceHolder.offset;
                }
            }
            int capacity = Math.min(this.getMaxLength() - replaceHolder.offset, Math.max(a, replaceHolder.length));
            int i = 0;
            int n = 0;
            while (i < capacity) {
                if (n < a && this.isValidCharacter(replaceHolder.offset + i, text.charAt(n))) {
                    int char1 = text.charAt(n);
                    if (char1 != this.getCharacter(replaceHolder.offset + i, (char)char1) && o == null) {
                        o = new StringBuilder();
                        if (n > 0) {
                            o.append(text.substring(0, n));
                        }
                    }
                    if (o != null) {
                        o.append(this.getCharacter(replaceHolder.offset + i, (char)char1));
                    }
                    ++n;
                }
                else if (this.isLiteral(replaceHolder.offset + i)) {
                    if (o != null) {
                        o.append(this.getLiteral(replaceHolder.offset + i));
                        if (n < a) {
                            capacity = Math.min(capacity + 1, this.getMaxLength() - replaceHolder.offset);
                        }
                    }
                    else if (n > 0) {
                        o = new StringBuilder(capacity);
                        o.append(text.substring(0, n));
                        o.append(this.getLiteral(replaceHolder.offset + i));
                        if (n < a) {
                            capacity = Math.min(capacity + 1, this.getMaxLength() - replaceHolder.offset);
                        }
                        else if (replaceHolder.cursorPosition == -1) {
                            replaceHolder.cursorPosition = replaceHolder.offset + i;
                        }
                    }
                    else {
                        ++replaceHolder.offset;
                        --replaceHolder.length;
                        --i;
                        --capacity;
                    }
                }
                else if (n >= a) {
                    if (o == null) {
                        o = new StringBuilder();
                        if (text != null) {
                            o.append(text);
                        }
                    }
                    o.append(this.getPlaceholderCharacter());
                    if (a > 0 && replaceHolder.cursorPosition == -1) {
                        replaceHolder.cursorPosition = replaceHolder.offset + i;
                    }
                }
                else {
                    return false;
                }
                ++i;
            }
            if (o != null) {
                replaceHolder.text = o.toString();
            }
            else if (text != null && replaceHolder.offset + a > this.getMaxLength()) {
                replaceHolder.text = text.substring(0, this.getMaxLength() - replaceHolder.offset);
            }
            if (this.getOverwriteMode() && replaceHolder.text != null) {
                replaceHolder.length = replaceHolder.text.length();
            }
        }
        return super.canReplace(replaceHolder);
    }
    
    static {
        EmptyMaskChars = new MaskFormatter.MaskCharacter[0];
    }
    
    private class MaskCharacter
    {
        public boolean isLiteral() {
            return false;
        }
        
        public boolean isValidCharacter(char c) {
            if (this.isLiteral()) {
                return this.getChar(c) == c;
            }
            int char1 = this.getChar(c);
            String validCharacters = MaskFormatter.this.getValidCharacters();
            if (validCharacters != null && validCharacters.indexOf(char1) == -1) {
                return false;
            }
            String invalidCharacters = MaskFormatter.this.getInvalidCharacters();
            return invalidCharacters == null || invalidCharacters.indexOf(char1) == -1;
        }
        
        public char getChar(char c) {
            return c;
        }
        
        public void append(StringBuilder sb, String s, int[] array, String s2) throws ParseException {
            int n = array[0] < s.length();
            int n2 = n ? s.charAt(array[0]) : '\0';
            if (this.isLiteral()) {
                sb.append(this.getChar((char)n2));
                if (MaskFormatter.this.getValueContainsLiteralCharacters()) {
                    if (n && n2 != this.getChar((char)n2)) {
                        throw new ParseException("Invalid character: " + (char)n2, array[0]);
                    }
                    ++array[0];
                }
            }
            else if (array[0] >= s.length()) {
                if (s2 != null && array[0] < s2.length()) {
                    sb.append(s2.charAt(array[0]));
                }
                else {
                    sb.append(MaskFormatter.this.getPlaceholderCharacter());
                }
                ++array[0];
            }
            else if (this.isValidCharacter((char)n2)) {
                sb.append(this.getChar((char)n2));
                ++array[0];
            }
            else {
                throw new ParseException("Invalid character: " + (char)n2, array[0]);
            }
        }
    }
    
    private class LiteralCharacter extends MaskFormatter.MaskCharacter
    {
        private char fixedChar;
        
        public LiteralCharacter(char fixedChar) {
            this.fixedChar = fixedChar;
        }
        
        @Override
        public boolean isLiteral() {
            return true;
        }
        
        @Override
        public char getChar(char c) {
            return this.fixedChar;
        }
    }
    
    private class DigitMaskCharacter extends MaskFormatter.MaskCharacter
    {
        @Override
        public boolean isValidCharacter(char ch) {
            return Character.isDigit(ch) && super.isValidCharacter(ch);
        }
    }
    
    private class UpperCaseCharacter extends MaskFormatter.MaskCharacter
    {
        @Override
        public boolean isValidCharacter(char ch) {
            return Character.isLetter(ch) && super.isValidCharacter(ch);
        }
        
        @Override
        public char getChar(char ch) {
            return Character.toUpperCase(ch);
        }
    }
    
    private class LowerCaseCharacter extends MaskFormatter.MaskCharacter
    {
        @Override
        public boolean isValidCharacter(char ch) {
            return Character.isLetter(ch) && super.isValidCharacter(ch);
        }
        
        @Override
        public char getChar(char ch) {
            return Character.toLowerCase(ch);
        }
    }
    
    private class AlphaNumericCharacter extends MaskFormatter.MaskCharacter
    {
        @Override
        public boolean isValidCharacter(char ch) {
            return Character.isLetterOrDigit(ch) && super.isValidCharacter(ch);
        }
    }
    
    private class CharCharacter extends MaskFormatter.MaskCharacter
    {
        @Override
        public boolean isValidCharacter(char ch) {
            return Character.isLetter(ch) && super.isValidCharacter(ch);
        }
    }
    
    private class HexCharacter extends MaskFormatter.MaskCharacter
    {
        @Override
        public boolean isValidCharacter(char c) {
            return (c == '0' || c == '1' || c == '2' || c == '3' || c == '4' || c == '5' || c == '6' || c == '7' || c == '8' || c == '9' || c == 'a' || c == 'A' || c == 'b' || c == 'B' || c == 'c' || c == 'C' || c == 'd' || c == 'D' || c == 'e' || c == 'E' || c == 'f' || c == 'F') && super.isValidCharacter(c);
        }
        
        @Override
        public char getChar(char c) {
            if (Character.isDigit(c)) {
                return c;
            }
            return Character.toUpperCase(c);
        }
    }
}
